package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.SOAPTags;
import dk.sosi.seal.model.constants.SubjectRelations10Tags;
import dk.sosi.seal.xml.TagUtil;
import dk.sosi.seal.xml.XmlUtil;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.Properties;

import static org.junit.Assert.*;

public class SubjectRelations10Test {

    @Test
    public void subjectRelations10DomBuilder_parentalCustodyHolder_containsCorrectValues() {
        // Arrange
        String type = "parentalCustodyHolder";
        String cpr = "0303033333";
        String soapXml = getSoapXml(cpr, type);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations10Tags.SUBJECT_RELATIONS);
        SubjectRelations10Impl subjectRelations10Impl = new SubjectRelations10Impl(elm);

        // Act
        SubjectRelations10DomBuilder subjectRelations10DomBuilder = new SubjectRelations10DomBuilder(subjectRelations10Impl);
        Document build = subjectRelations10DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
    }

    @Test
    public void subjectRelations10DomBuilder_partlyWardCustodyHolder_containsCorrectValues() {
        // Arrange
        String type = "partlyWardCustodyHolder";
        String cpr = "0303034444";
        String soapXml = getSoapXml(cpr, type);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations10Tags.SUBJECT_RELATIONS);
        SubjectRelations10Impl subjectRelations10Impl = new SubjectRelations10Impl(elm);

        // Act
        SubjectRelations10DomBuilder subjectRelations10DomBuilder = new SubjectRelations10DomBuilder(subjectRelations10Impl);
        Document build = subjectRelations10DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
    }

    @Test
    public void subjectRelations10DomBuilder_wardCustodyHolder_containsCorrectValues() {
        // Arrange
        String type = "wardCustodyHolder";
        String cpr = "0303035555";
        String soapXml = getSoapXml(cpr, type);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations10Tags.SUBJECT_RELATIONS);
        SubjectRelations10Impl subjectRelations10Impl = new SubjectRelations10Impl(elm);

        // Act
        SubjectRelations10DomBuilder subjectRelations10DomBuilder = new SubjectRelations10DomBuilder(subjectRelations10Impl);
        Document build = subjectRelations10DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
    }

    @Test
    public void subjectRelations10Impl_emptyVerifiedRelation_validationError() {
        // Arrange
        String soapXml = getEmptySoapXml();
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations10Tags.SUBJECT_RELATIONS);

        // Act
        try {
            new SubjectRelations10Impl(elm);
            fail("Should have thrown exception");
        } catch (ModelException e) {
            // Assert
            assertTrue(e.getMessage().contains("Error validating subject relations"));
        }
    }

    private static String getEmptySoapXml() {
        return  "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                "  <soapenv:Header />\n" +
                "  <soapenv:Body>\n" +
                "    <srp:SubjectRelations xmlns:srp=\"urn:dk:healthcare:saml:subject_relations_profile:1.0\">\n" +
                "    </srp:SubjectRelations>\n" +
                "  </soapenv:Body>\n" +
                "</soapenv:Envelope>";
    }

    private static String getSoapXml(String personId, String relatedType) {
        final String subjectRelationsXml =
                    "<srp:SubjectRelations xmlns:srp=\"urn:dk:healthcare:saml:subject_relations_profile:1.0\">\n" +
                    "   <srp:VerifiedRelation\n" +
                    "      relationType=\""+relatedType+"\"\n" +
                    "      relatedPersonID=\""+personId+"\"\n" +
                    "      relatedPersonIDType=\"URN:OID:1.2.208.176.1.2\"/>\n" +
                    "</srp:SubjectRelations>\n";

        return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                    "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                    "  <soapenv:Header />\n" +
                    "  <soapenv:Body>\n" +
                        subjectRelationsXml +
                    "  </soapenv:Body>\n" +
                    "</soapenv:Envelope>";

    }

}